@@ -62,6 +62,10 @@
 #define OPT_NO_UPSTREAM_CONNECTION       11
 #define OPT_REGISTER_WAIT                12
 #define OPT_CLUSTER                      13
+#define OPT_LOG_TO_FILE                  14
+#define OPT_UPSTREAM_CONNINFO            15
+#define OPT_NO_CONNINFO_PASSWORD         16
+#define OPT_REPLICATION_USER             17
 
 /* deprecated command line options */
 #define OPT_INITDB_NO_PWPROMPT           998
@@ -80,10 +84,11 @@ typedef struct
 	bool		verbose;
 	bool		terse;
 	bool		force;
+	char		pg_bindir[MAXLEN]; /* overrides setting in repmgr.conf */
 
-	/* options which override setting in repmgr.conf */
-	char		loglevel[MAXLEN];
-	char		pg_bindir[MAXLEN];
+	/* logging parameters */
+	char		loglevel[MAXLEN];  /* overrides setting in repmgr.conf */
+	bool		log_to_file;
 
 	/* connection parameters */
 	char		dbname[MAXLEN];
@@ -105,13 +110,18 @@ typedef struct
 	bool		fast_checkpoint;
 	bool		without_barman;
 	bool		no_upstream_connection;
+	bool		no_conninfo_password;
 	bool		copy_external_config_files;
 	int			copy_external_config_files_destination;
-	bool		wait_register_sync;
-	int			wait_register_sync_seconds;
+	char		upstream_conninfo[MAXLEN];
+	char		replication_user[MAXLEN];
 
 	char		recovery_min_apply_delay[MAXLEN];
 
+	/* standby register paarameters */
+	bool		wait_register_sync;
+	int			wait_register_sync_seconds;
+
 	/* witness create parameters */
 	bool		witness_pwprompt;
 
@@ -140,15 +150,17 @@ typedef struct
 
 #define T_RUNTIME_OPTIONS_INITIALIZER { \
 		/* general repmgr options */	\
-		"", false, false, false,		\
-		/* options which override setting in repmgr.conf */ \
-		"", "",                         \
+		"", false, false, false, "",	\
+		/* logging parameters */ \
+		"", false,                      \
 		/* connection parameters */		\
 		"", "", "", "", "", "", "", 	\
 		false, false, false,		    \
 		/* standby clone parameters */  \
 		false, DEFAULT_WAL_KEEP_SEGMENTS, false, false, false, false, false, false, \
-		CONFIG_FILE_SAMEPATH, false, 0, "", \
+		false, CONFIG_FILE_SAMEPATH, "", "", "", \
+		/* standby register paarameters */ \
+	    false, 0,							 \
 		/* witness create parameters */ \
 		false,                          \
 		/* standby follow parameters */ \
