@@ -90,31 +90,18 @@ maxlen_snprintf(char *str, const char *format,...)
 
 
 /*
- * Adapted from: src/fe_utils/string_utils.c
- *
- * Function not publicly available before PostgreSQL 9.6.
+ * Escape a string for use as a parameter in recovery.conf
+ * Caller must free returned value
  */
-void
-appendShellString(PQExpBuffer buf, const char *str)
+char *
+escape_recovery_conf_value(const char *src)
 {
-	const char *p;
+	char	   *result = escape_single_quotes_ascii(src);
 
-	appendPQExpBufferChar(buf, '\'');
-	for (p = str; *p; p++)
+	if (!result)
 	{
-		if (*p == '\n' || *p == '\r')
-		{
-			fprintf(stderr,
-					_("shell command argument contains a newline or carriage return: \"%s\"\n"),
-					str);
-			exit(ERR_BAD_CONFIG);
-		}
-
-		if (*p == '\'')
-			appendPQExpBufferStr(buf, "'\"'\"'");
-		else
-			appendPQExpBufferChar(buf, *p);
+		fprintf(stderr, _("%s: out of memory\n"), progname());
+		exit(ERR_INTERNAL);
 	}
-
-	appendPQExpBufferChar(buf, '\'');
+	return result;
 }
